/**
 * This file is part of the equanda project.
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific language governing rights and
 * limitations under the License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package myapp.example.gui.services;

import javassist.runtime.Desc;
import org.apache.tapestry.Translator;
import org.apache.tapestry.ioc.Configuration;
import org.apache.tapestry.ioc.MappedConfiguration;
import org.apache.tapestry.ioc.OrderedConfiguration;
import org.apache.tapestry.ioc.ServiceBinder;
import org.apache.tapestry.ioc.annotations.InjectService;
import org.apache.tapestry.ioc.services.Coercion;
import org.apache.tapestry.ioc.services.CoercionTuple;
import org.apache.tapestry.services.Request;
import org.apache.tapestry.services.RequestFilter;
import org.apache.tapestry.services.RequestHandler;
import org.apache.tapestry.services.Response;
import org.slf4j.Logger;

import java.io.IOException;

/**
 * This module is automatically included as part of the Tapestry IoC Registry, it's a good place to configure and extend
 * Tapestry, or to place your own service definitions.
 */
public class AppModule
{
    /**
     * Bind extra services.
     *
     * @param binder object to bind services to
     */
    public static void bind( ServiceBinder binder )
    {
        // binder.bind(MyServiceInterface.class, MyServiceImpl.class);

        // Make bind() calls on the binder object to define most IoC services.
        // Use service builder methods (example below) when the implementation
        // is provided inline, or requires more initialization than simply
        // invoking the constructor.

        // assure that the ContextClassLoader is used by javassist (seems to not be the default)
        System.out.println( "Desc.useContextClassLoader = " + Desc.useContextClassLoader );
        javassist.runtime.Desc.useContextClassLoader = true;
    }

    /**
     * Contribute application defaults
     *
     * @param configuration configuration to add to
     */
    public static void contributeApplicationDefaults(
        MappedConfiguration<String, String> configuration )
    {
        // Contributions to ApplicationDefaults will override any contributions to
        // FactoryDefaults (with the same key). Here we're restricting the supported
        // locales to just "en" (English). As you add localised message catalogs and other assets,
        // you can extend this list of locales (it's a comma seperated series of locale names;
        // the first locale name is the default when there's no reasonable match).

        configuration.add( "tapestry.supported-locales", "en,nl" );
    }


    /**
     * This is a service definition, the service will be named "TimingFilter". The interface, RequestFilter, is used
     * within the RequestHandler service pipeline, which is built from the RequestHandler service configuration.
     * Tapestry IoC is responsible for passing in an appropriate Log instance. Requests for static resources are handled
     * at a higher level, so this filter will only be invoked for Tapestry related requests.
     * <p/>
     * <p/>
     * Service builder methods are useful when the implementation is inline as an inner class (as here) or require some
     * other kind of special initialization. In most cases, use the static bind() method instead.
     * <p/>
     * <p/>
     * If this method was named "build", then the service id would be taken from the service interface and would be
     * "RequestFilter".  Since Tapestry already defines a service named "RequestFilter" we use an explicit service id
     * that we can reference inside the contribution method.
     *
     * @param log log
     * @return request filter
     */
    public RequestFilter buildTimingFilter( final Logger log )
    {
        return new RequestFilter()
        {
            public boolean service( Request request, Response response, RequestHandler handler )
                throws IOException
            {
                long startTime = System.currentTimeMillis();

                try
                {
                    // The reponsibility of a filter is to invoke the corresponding method
                    // in the handler. When you chain multiple filters together, each filter
                    // received a handler that is a bridge to the next filter.

                    return handler.service( request, response );
                }
                finally
                {
                    long elapsed = System.currentTimeMillis() - startTime;

                    log.info( String.format( "Request time: %d ms", elapsed ) );
                }
            }
        };
    }

    /**
     * This is a contribution to the RequestHandler service configuration. This is how we extend Tapestry using the
     * timing filter. A common use for this kind of filter is transaction management or security.
     *
     * @param configuration configuration to add to
     * @param filter filter info
     */
    public void contributeRequestHandler( OrderedConfiguration<RequestFilter> configuration,
                                          @InjectService( "TimingFilter" )
                                          RequestFilter filter )
    {
        // Each contribution to an ordered configuration has a name, When necessary, you may
        // set constraints to precisely control the invocation order of the contributed filter
        // within the pipeline.

        configuration.add( "Timing", filter );
    }
}
